#!/usr/bin/perl
#
# Perl version of the Telnet blinkenlight server
# (c) 2003 Wolfgang Dautermann
#
# Idea (and original in C) from Rene Schickbauer

use warnings;
use strict ;
use diagnostics;
use English;  # Support variables like "$EFFECTIVE_USER_ID", etc.
use Term::ANSIScreen qw/:color :cursor :screen/;
use Time::HiRes qw(usleep) ;
use IO::Socket;
use Getopt::Long;

#
# Configure these variables as you like
#
my $PORT = 10001 ;        # On which port should the server listen?

# be sure to modify ${MOVIEDIR} in the Makefile
# (if you want to use "make install"), if you modify it here!
my $moviedir = "/usr/local/share/blinkenperl/movies" ; # where to find the movies
my $blmext = "blm" ;     # extensions of blm files in the movie directory


my $urcol = 3 ; my $urlin = 2 ; # upper right column/line
                                # where the movie in the screen can be
                                # displayed (3rd column, 2nd line)
my $screen=
    "/|==================|\\\n" .
    "||                  ||\n" .
    "||                  ||\n" .
    "||                  ||\n" .
    "||                  ||\n" .
    "||                  ||\n" .
    "||                  ||\n" .
    "||                  ||\n" .
    "||                  ||\n" .
    "|| HAUS DES LEHRERS ||\n" .
    "||                  ||\n" .
    "|| Blinken       _  ||\n" .
    "||    lights    | | ||\n" .
    "||              | | ||\n" .
    "======================\n" ;

# Parse command line options
GetOptions( "port=i" => \$PORT ) ;


$SIG{CHLD} = 'IGNORE'; # do not create Zombie-Processes
my $server = IO::Socket::INET->new( Proto  => 'tcp',
				    LocalPort => $PORT,
				    Listen    => SOMAXCONN,
				    Reuse     => 1);
die "can't setup server" unless $server;

# Drops privs
# if you start the server as user root (which should not be necessary,
# (and should not be done), if you do not want to use a privileged (<1024)
# Port (eg. Port 23 - telnet).
#
# Adjust the User/Group if you like...
my $uid = (getpwnam("nobody"))[2];
my $gid = (getgrnam("nogroup"))[2];
($REAL_GROUP_ID,$EFFECTIVE_GROUP_ID) = ($gid,$gid);
($REAL_USER_ID,$EFFECTIVE_USER_ID) = ($uid,$uid);

print "[Server $0 accepting telnet clients at port $PORT]\n";

while (my $client = $server->accept()) {
    my $pid = fork();
    die "Cannot fork" unless defined $pid;
    do { close $client; next; } if $pid; # parent
    # fall through in child

    my $movieline ;
    my @filelist = glob("$moviedir/*.$blmext") ;
    my $randomfile = $filelist[int(rand($#filelist))] ;

    printf "[Connect from %s] Playing file %s \n", $client->peerhost, $randomfile ;

    my ($author,$name,$description,@moviearray) = readblm($randomfile) ;
    print $client cls(), locate(0,0) ;
    printf $client "%s \n",$screen ;
    if ($author ne "") { print $client "Author: $author" ; } ;
    if ($name ne "") { printf $client "Name: $name" ; } ;
    if ($description ne "") { print $client "Description: $description" ; } ;
    my $l = 0 ;
    my $time = 0 ;
    foreach $movieline (@moviearray) {
	if ($movieline =~ /^@[0-9]*/) {
	    usleep($time*1000);
	    $time = substr($movieline,1)  ;
	    $l = 0 ;
	} else {
	    print $client locate($urlin+$l,$urcol) ;
	    $movieline =~ s/0/\ /g ; $movieline =~ s/1/\#/g ;
	    print $client $movieline ;
	    $l ++ ;
	} ;
    }
    print $client locate(17+3,0); # +3 for auhor,name,description
    close $client;
    exit ;
}

# read a blm-File, strip comments and empty lines
# return Moviearray
sub readblm {
    my ($filename) = @_ ; # parameters
    my (@moviearray,$i,$line,$author,$name,$description) ;
    open(FILE, "< $filename") or die "Can not open file: $filename \n";
    $i = 0 ;
    $author = ""; $name="" ; $description="";  
    while (<FILE>) {
	$line = $_ ;
	if ($line =~ /^#.*$/) { # Comment
	    # parse Comments in blm-Files (author, category, etc.)
	    if (substr($line,0,11) eq "# author = ") { $author=substr($line,11); }
	    if (substr($line,0,9) eq "# name = ") {$name=substr($line,9); }
	    if (substr($line,0,16) eq "# description = ") {$description=substr($line,16); }
	    next
	} ; # ignore/parse comments
	if ($line =~ /^$/) {
	    next ;
	}; # ignore blank lines
	$moviearray[$i++] = $line ;
    }
    return ($author,$name,$description,@moviearray) ;
}

=pod

=head1 NAME

blinkserver.pl - A telnet server that plays Blinkenlights movies.

=head1 SYNOPSIS

blinkserver.pl [--port=number]


=head1 OPTIONS

=over 8

=item B<--port>=I<number>: On which Port should the server listen?

=head1 DESCRIPTION

B<This program> will listen on the defined port (current default value: 10001)
and play a random Blinkenlights-Movie if a telnet client connects.


=head1 NOTES

Look for user configurable perl-variables in the skript, if you
want to modify other things...

=head1 AUTHOR

Wolfgang Dautermann, E<lt>dauti@oeh.tu-graz.ac.atE<gt>

=head1 SEE ALSO

http://www.blinkenlights.de

=cut

__END__


 TODO:
    - BML-Fileformat
    - Accept other than 18x8 Fileformats

 CHANGELOG:
    [2003-01-06] initial release
    [2003-01-10] Parse and output comments (author, etc.) in BLM-Files
    [2003-01-14] Add some documentation, translate some comments from german to english
    [2003-01-26] Add some code to drop privs if started as root.
    [2003-01-30] Fixed the code introduced 4 days ago,
                 Added code for parsing command line options.
